import { File } from "./File";
/**
 * Checks if given value is a File, Blob or file-look-a-like object.
 *
 * @param value A value to test
 */
export declare const isFile: (value: unknown) => value is File;
