export declare const FormData: {
    new (form?: HTMLFormElement | undefined): FormData;
    prototype: FormData;
}, Blob: {
    new (blobParts?: BlobPart[] | undefined, options?: BlobPropertyBag | undefined): Blob;
    prototype: Blob;
}, File: {
    new (fileBits: BlobPart[], fileName: string, options?: FilePropertyBag | undefined): File;
    prototype: File;
};
